#pragma once
#include <ESPMegaPRO.h>
#include <Nextion.h>
#include <PubSubClient.h>
#include <PubSubClientTools.h>
#include "espmega_iot_timer.hpp"
#include "espmega_iot_external_lcd.hpp"

// I/O Assignment
// Available Inputs 0-15
// Available Outputs 0-3, 7-11
// There are 4 rows of lights
// Each row has 2 lights
// Each light has 2 states
#define ROW1_LIGHT1 0
#define ROW1_LIGHT2 1
#define ROW2_LIGHT1 2
#define ROW2_LIGHT2 3
#define ROW3_LIGHT1 7
#define ROW3_LIGHT2 8
#define ROW4_LIGHT1 9
#define ROW4_LIGHT2 10
#define COMPUTER_DESK_CONTACTOR 12

// Bus Overclocking Configuration
// Do not enable if you are using external I/O cards as it will cause signal integrity issues.
// Choose only one mode
//#define OVERCLOCK_FM
//#define OVERCLOCK_FM2

// I/O Configuration
#define VIRTUAL_INTERRUPT_PRELOAD // Preload Virtual Interrupts buffer

// Enable Software Module(s)
// Deprecated. Use Build Flags instead.
// #define ENABLE_INTERNAL_LCD
// #define ENABLE_IR_MODULE
// #define ENABLE_CLIMATE_MODULE // Require IR Module
// #define ENABLE_ANALOG_MODULE
// #define ENABLE_WEBUI

// IR Kit Configuration
#define IR_RECIEVE_PIN 35
#define IR_SEND_PIN 5
#define MARK_EXCESS_MICROS 20
#define IR_RAW_BUFFER_LENGTH 750
#define AC_MAX_TEMPERATURE 30
#define AC_MIN_TEMPERATURE 15
#define DHT22_PIN 32

// External LCD Configuration
#define ENABLE_EXTERNAL_LCD
#define TXD2 4
#define RXD2 17

#ifdef ENABLE_EXTERNAL_LCD
#define ESPMega_EXTLCD Serial2
#endif

// Analog Module Configuration
#define ANALOG_REPORTING_INTERVAL 500

// User Defined Variables

// User Defined Functions
#define elcd ESPMega_EXTLCD
void elcd_send_stop_bit();
bool row_is_on(int light1_pin, int light2_pin);
bool row_is_on_or(int light1_pin, int light2_pin);
void update_lcd_row1();
void update_lcd_row2();
void update_lcd_row3();
void update_lcd_row4();
void update_lcd_ac();
void update_toggle_button();
void light_toggle_pop_callback(void *ptr);
void row1_lv1_pop_callback(void *ptr);
void row1_lv2_pop_callback(void *ptr);
void row2_lv1_pop_callback(void *ptr);
void row2_lv2_pop_callback(void *ptr);
void row3_lv1_pop_callback(void *ptr);
void row3_lv2_pop_callback(void *ptr);
void row4_lv1_pop_callback(void *ptr);
void row4_lv2_pop_callback(void *ptr);
void row1_master_pop_callback(void *ptr);
void row2_master_pop_callback(void *ptr);
void row3_master_pop_callback(void *ptr);
void row4_master_pop_callback(void *ptr);
void increase_temp();
void decrease_temp();
void temp_plus_pop_callback(void *ptr);
void temp_neg_pop_callback(void *ptr);
void fan_pop_callback(void *ptr);
void mode_pop_callback(void *ptr);
void ac_toggle_pop_callback(void *ptr);


// User Defined IoT Core Callback Functions (Required)
void user_mqtt_callback(char* topic, uint8_t topic_length, char* payload, unsigned int payload_length);
void user_state_request_callback();
void mqtt_connected_user_callback();
void user_pre_init();
void user_init();
void user_loop();
void pwm_changed_user_callback(int pin);
void ac_changed_user_callback(int mode, int temperature, int fan_speed);
void timer_tick_callback();
void virtual_interrupt_user_callback(int pin, int state);

// ESPMega IoT Core Build-in Functions
extern void pwm_set_state(int id, int state);
extern void pwm_set_value(int id, int value);
extern void pwm_toggle(int id);
extern void pwm_toggle(int id1, int id2);
extern void pwm_cycle_value(int id);
extern bool pwm_get_state(int id);
extern uint16_t pwm_get_value(int id);
extern boolean pwm_group_state(int id1, int id2);
extern bool input_get_state(int id);
extern bool standalone;
extern PubSubClient mqtt;

// IR Kit Build-in Functions
#ifdef ENABLE_IR_MODULE
extern void ac_set_state(int mode, int temperature, int fan_speed);
extern uint8_t ac_get_temperature();
extern uint8_t ac_get_mode();
extern uint8_t ac_get_fan_speed();
#endif

//Analog Expansion Card Build-in Functions
#ifdef ENABLE_ANALOG_MODULE
extern void dac_set_value(int id, int value);
extern void dac_set_state(int id, bool state);
extern void enable_adc(int id);
extern void disable_adc(int id);
extern void enable_dac(int id);
extern uint16_t adc_get_value(int id);
extern bool adc_get_state(int id);
extern uint16_t dac_get_value(int id);
extern bool dac_get_state(int id);
#endif